#!/bin/bash
# -lboost_program_options might need a package to be installed (part of
#   boost C++ libraries).
export LIBS_BASE="-lboost_program_options "

# note on math libraries:
#  o program assumes by default that a BLAS with 64bit integers is linked.
#    for MKL, that means using the mkl_intel_ilp64 variant (NOT the lp64 one!),
#    for ACML that means using the version from the _int64 subdirectory
#    (e.g., gfortran64_fma4_int64/)
#  o If a 64bit integer blas is not available, you can re-define "FORTINT"
#    from 'long' to 'int' at the top of CxFortranInt.h, or add -D_I4_ to
#    the compiler command line.
#  o using the -Wl,-rpath,, you can embed an "rpath" into the executable. This
#    directory is used by the dynamic linker to look up missing libraries.
#    This makes setting an LD_LIBRARY_PATH for BLAS unnecessary.
# The following lines are examples of BLAS/LAPACK lines you might take as
# suggestion on how to construct one for your own system:

# export LIBS_LA="-L/opt/acml5.1.0/gfortran64_int64/lib/ -lacml -Wl,-rpath,/opt/acml5.1.0/gfortran64_int64/lib/"
# export LIBS_LA="-L/opt/intel/mkl/10.2.4.032/lib/em64t/ -lmkl_core -lmkl_sequential -lmkl_intel_ilp64 -Wl,-rpath,/opt/intel/mkl/10.2.4.032/lib/"
# export LIBS_LA="-L/opt/intel/composerxe/mkl/lib/intel64 -lmkl_gf_ilp64 -lmkl_core -lmkl_sequential -Wl,-rpath,/opt/intel/composerxe/mkl/lib/intel64"
export LIBS_LA="-L$MKLROOT/lib/intel64 -lmkl_gf_ilp64 -lmkl_core -lmkl_sequential -pthread -Wl,-rpath,$MKLROOT/lib/intel64"

export OTHER="-DUSE_CTINT1E_H"

# compile ir/wmme with optimizations
c++ -o wmme -O3 -DNDEBUG $OTHER src/*.cpp $LIBS_BASE $LIBS_LA


# if c++ returned 0, most likely everything went right. Print a comment to say so.
if [ $? -eq 0 ]; then
   echo " Build finished. Try running './wmme' or 'python rhf_in_python_via_wmme_module.py'."
fi
