# Copyright (c) 2012  Gerald Knizia
# 
# This file is part of the IR/WMME program
# (See https://sites.psu.edu/knizia/)
# 
# IR/WMME is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3.
# 
# IR/WMME is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with bfint (LICENSE). If not, see http://www.gnu.org/licenses/

#
# This is an example for using the 'wmme.py' interface to wmme.
# This should work if invoked at the shell with either
#
#    export PYTHONPATH=$HOME/dev/wmme:$PYTHONPATH
#    python wmme_module_example.py
#
# or
#
#    PYTHONPATH=$HOME/dev/wmme:$PYTHONPATH python wmme_module_example.py
#
# where the path (here $HOME/dev/wmme) refers to the actual path in which
# wmme was unpacked/installed.

import numpy as np
import wmme
from os import path

np.set_printoptions(linewidth=1000,precision=5)

# read an xyz file and make an FAtomSet object.
Pos,Elements = wmme.ReadXyzFile("benzene.xyz")
Atoms = wmme.FAtomSet(Pos, Elements)

# create an integral context (ic) to make integrals for it.
BasisLibs = ["def2-nzvpp-jkfit.libmol", "def2-nzvpp-orb.libmol"]
ic = wmme.FIntegralContext(Atoms, "def2-SVP", FitBasis="univ-JKFIT", BasisLibs=BasisLibs)

# compute the overlap matrix of the given molecule in the def2-SVP basis.
S = ic.MakeOverlap()
print "Overlap matrix:\n%s" % S
